# forward selection

#install.packages("MASS")
#install.packages("lars")
library(MASS)
library(lars)

#diry = "C:/Users/Sangwon/Dropbox/CMU/CURRENT courses/JournalClub/MyNotes/Knockoff/"
#setwd(diry)
source('funs.R')

n = 300
p = 100
k = 30
b = 10

## Beta is fixed in our single model
x = matrix(nrow = n, ncol = p)
for(irow in 1:n){
  for(icol in 1:p){
    x[irow, icol] = rnorm(1,0,1)
  }
}
colsize = sapply(1:p, function(j) sqrt(sum(x[,j]^2)) ,simplify = "array")
x = scale(x, center= FALSE, scale = colsize)


signal_beta = c()
for(ki in 1:k)   signal_beta = c(signal_beta, (if(runif(1,0,1)>.5) b else -b))
beta = rep(0,p)
beta[1:k] = signal_beta #beta[sample(1:1000, 30)] <- signal_beta

# observed_FDR = numeric(100)

#for each run, devise 100
# for(runs_i in 1:100){
#cat("run: ", runs_i, fill=TRUE)

observed_FDR = numeric(50)
observed_power = numeric(50)

for(b in c(1,5,10,20)){
  cat("b: ",b,fill=TRUE)
  ## Beta is fixed in our single model
  signal_beta = c()
  for(ki in 1:k)   signal_beta = c(signal_beta, (if(runif(1,0,1)>0.5) b else b))
  beta = rep(0,p)
  beta[1:k] = signal_beta #beta[sample(1:1000, 30)] <- signal_beta
  
  
  for(runs_i in 1:50){

  noise = rnorm(n=n,mean=0,sd=1)
  y = x%*%beta + noise
  xt = knockoff(x)

# Do forward selection
  X = data.frame(cbind(x,xt))
  lm1 = lm(y~1, data = X)
  lm_biggest = lm(y~., data = X)
  biggest <- formula(lm_biggest)
  fstep1 = step(lm1, direction='forward',scope=biggest, steps = 1000,trace=F)
  entry = as.numeric(substr(fstep1$anova[-1,1],start=4,stop=100))
  entry = c(entry,  (1:200)[!(1:200 %in% entry)]  )
  entry = abs(entry)
  ## 3. Obtain Z and W statistics
  Z = 2*p+1-order(entry)
  W = numeric(p)
  for(j in 1:p)     W[j] = max(Z[j],Z[j+p])*sign(Z[j]-Z[j+p])
  #Checking results
  fdr_target = .2
  tlist = unique(abs(W))
  fdr_est_knockoff = numeric(length(tlist))
  #fdr_est_knockoffplus = numeric(length(tlist))
  
  for(j in 1:length(tlist)){
    tee = tlist[j]
    fdr_est_knockoff[j] = (sum( W < (-tee))) / max( sum(W >= tee), 1)
    #fdr_est_knockoffplus[j] = (1+sum( W < (-tee), na.rm = TRUE)) / max( sum(W >= tee, na.rm = TRUE), 1)
  } 
  #Thresh = min(tlist[fdr_est_knockoffplus<fdr_target])
  Thresh = min(tlist[fdr_est_knockoff<fdr_target])
  
  #Checking observed FDR for our dataset
  selected = (1:p)[W >= Thresh]
  num_nulls_selected = sum(!( selected %in% (1:k)))
  num_selected = length(selected)
  observed_FDR[runs_i] = num_nulls_selected/num_selected
  
  num_signals_selected = sum( selected %in% (1:k))
  observed_power[runs_i] = num_signals_selected/k

  #plotting
  pdf(paste0("Knockoff_FS_",b,".pdf"), height = 7, width = 14)
  par(mfrow = c(1,2))
  
  hist(observed_FDR, main = paste0("FDR from simulations \n n=300,p=100 \n k=30,b=",b), col = 'skyblue')
  abline(v = mean(observed_FDR, na.rm=T), col = 'red', lwd = 2)
  legend("topleft", col='red', lwd = 2, legend = round(mean(observed_FDR,na.rm=T),3))
  
  hist(observed_power,main = paste0("Power from simulations \n n=300,p=100 \n k=30,b=",b), col = 'skyblue')
  abline(v = mean(observed_power,na.rm=T), col = 'red', lwd = 2)
  legend("topleft", col='red', lwd = 2, legend = round(mean(observed_power,na.rm=T),3))
  dev.off()
}
}
