# forward selection
library(MASS)
library(lars)

#diry = "C:/Users/Sangwon/Dropbox/CMU/CURRENT courses/JournalClub/MyNotes/Knockoff/"
#setwd(diry)
source('funs.R')

n = 300
p = 100
k = 30
nruns = 50

x = genX(n,p)

observed_FDR = numeric(nruns)
observed_FDR_plus = numeric(nruns)
observed_power = numeric(nruns)
observed_power_plus = numeric(nruns)

for(b in 1){#c(1,2,5,10,20)){
  cat("b: ",b,fill=TRUE)  
  beta = genbeta(k,b)  ## Beta is fixed in our single model
   
  for(runs_i in 1:nruns){  
    cat("run:",runs_i,fill=T)
    y = genY(x,beta)
    xt = knockoff(x)
    
    # Do forward selection
    X = data.frame(cbind(x,xt))
    lm1 = lm(y~1, data = X)
    lm_biggest = lm(y~., data = X)
    biggest <- formula(lm_biggest)
    fstep1 = step(lm1, direction='forward',scope=biggest, steps = 1000,trace=F)
    entry = as.numeric(substr(fstep1$anova[-1,1],start=4,stop=100))
    entry = c(entry,  (1:200)[!(1:200 %in% entry)]  )
    entry = abs(entry)
    
    ## Obtain Z and W statistics
    Z = 2*p+1-order(entry)
    W = numeric(p)
    for(j in 1:p)   W[j] = max(Z[j],Z[j+p])*sign(Z[j]-Z[j+p])
    
    #Checking results
    fdr_target = .2
    tlist = unique(abs(W))
    fdr_est_knockoff = numeric(length(tlist))
    fdr_est_knockoffplus = numeric(length(tlist))
    
    for(j in 1:length(tlist)){
      tee = tlist[j]
      fdr_est_knockoff[j] = (sum( W < (-tee))) / max( sum(W >= tee), 1)
      fdr_est_knockoffplus[j] = (1+sum( W < (-tee), na.rm = TRUE)) / max( sum(W >= tee, na.rm = TRUE), 1)
    } 
    ThreshPlus = min(tlist[fdr_est_knockoffplus<fdr_target])
    Thresh = min(tlist[fdr_est_knockoff<fdr_target])
  
    #Checking observed knockoff FDR for our dataset
    selected = (1:p)[W >= Thresh]
    num_nulls_selected = sum(!( selected %in% (1:k)))
    num_selected = length(selected)
    observed_FDR[runs_i] = num_nulls_selected/num_selected
    num_signals_selected = sum( selected %in% (1:k))
    observed_power[runs_i] = num_signals_selected/k
    
    #Checking observed knockoff+ FDR for our dataset
    selected = (1:p)[W >= ThreshPlus]
    num_nulls_selected = sum(!( selected %in% (1:k)))
    num_selected = length(selected)
    observed_FDR_plus[runs_i] = num_nulls_selected/num_selected
    num_signals_selected = sum( selected %in% (1:k))
    observed_power_plus[runs_i] = num_signals_selected/k
  }   
    
    #plotting knockoff
    pdf(paste0("Knockoff_FS_",b,".pdf"), height = 7, width = 14)
      par(mfrow = c(1,2))
      hist(observed_FDR, main = paste0("FDR from simulations \n n=300,p=100 \n k=30,b=",b), col = 'skyblue')
      abline(v = mean(observed_FDR, na.rm=T), col = 'red', lwd = 2)
      legend("topleft", col='red', lwd = 2, legend = round(mean(observed_FDR,na.rm=T),3))
      
      hist(observed_power,main = paste0("Power from simulations \n n=300,p=100 \n k=30,b=",b), col = 'skyblue')
      abline(v = mean(observed_power,na.rm=T), col = 'red', lwd = 2)
      legend("topleft", col='red', lwd = 2, legend = round(mean(observed_power,na.rm=T),3))
    dev.off()
    
    #plotting knockoff+
    pdf(paste0("Knockoff_FS+_",b,".pdf"), height = 7, width = 14)
      par(mfrow = c(1,2))
      hist(observed_FDR_plus, main = paste0("FDR from simulations \n n=300,p=100 \n k=30,b=",b), col = 'skyblue')
      abline(v = mean(observed_FDR, na.rm=T), col = 'red', lwd = 2)
      legend("topleft", col='red', lwd = 2, legend = round(mean(observed_FDR,na.rm=T),3))
      
      hist(observed_power_plus,main = paste0("Power from simulations \n n=300,p=100 \n k=30,b=",b), col = 'skyblue')
      abline(v = mean(observed_power,na.rm=T), col = 'red', lwd = 2)
      legend("topleft", col='red', lwd = 2, legend = round(mean(observed_power,na.rm=T),3))
    dev.off()    
  }
}
